// testwavein.cpp : Defines the entry point for the application.
//

#include "windows.h"
#include <stdio.h>
// #define SAMPLE_RATE 11025
// #define SAMPLE_RATE 8000
#define SAMPLE_RATE 44100

#pragma warning(disable: 4100)

#define CheckError(_x_,_y_) { if (MMSYSERR_NOERROR!=_x_) { printf("ERROR -> 0x%x = %s\r\n", _x_, _y_); \
														RETAILMSG(1,(TEXT("ERROR -> 0x%x = %S\r\n"), _x_, _y_)); } }


bool Record(char *buffer, int NumSamples)
{
   HWAVEIN WaveInHandle;
   WAVEFORMATEX WaveFormat;
   MMRESULT MMResult;
   WAVEHDR WaveHeader;
   MMTIME MMTime;


   // Setup the WaveFormat structure and open the Input device
   WaveFormat.wFormatTag =		WAVE_FORMAT_PCM;
   WaveFormat.nChannels =		2;
   WaveFormat.wBitsPerSample =	16;
   WaveFormat.nSamplesPerSec =	SAMPLE_RATE;
   WaveFormat.nBlockAlign =		( WaveFormat.nChannels * WaveFormat.wBitsPerSample ) / 8;
   WaveFormat.nAvgBytesPerSec = WaveFormat.nBlockAlign * WaveFormat.nSamplesPerSec;
   WaveFormat.cbSize =			0;

   MMResult = waveInOpen(&WaveInHandle, WAVE_MAPPER, &WaveFormat, 0, 0, CALLBACK_EVENT);
   CheckError(MMResult, "Failed to open input");

   
   // Setup the WaveHeader, whcih contaions info about the data to record
   memset(&WaveHeader, 0, sizeof(WaveHeader));
   WaveHeader.lpData = buffer;
   WaveHeader.dwBufferLength = NumSamples*4;
   WaveHeader.dwLoops = 1;
   WaveHeader.dwFlags = WHDR_BEGINLOOP | WHDR_ENDLOOP;

   MMResult = waveInPrepareHeader(WaveInHandle, &WaveHeader, sizeof(WaveHeader));
   CheckError(MMResult, "Failed to prepare header");

   // Tell the system to use our buffer
   MMResult = waveInAddBuffer(WaveInHandle, &WaveHeader, sizeof(WaveHeader));
   CheckError(MMResult, "Failed to add buffer");

   // Start recording
   MMResult = waveInStart(WaveInHandle);
   CheckError(MMResult, "Failed to Start");

   // Wait for user key or end of input
   printf("Recording ");
   fflush(stdout);
   do
   {
     Sleep(100);
   }
   while(!(WaveHeader.dwFlags & WHDR_DONE));

   // Get the position at which the recording stopped
   MMTime.wType = TIME_BYTES;
   MMResult = waveInGetPosition(WaveInHandle, &MMTime, sizeof(MMTime));
   CheckError(MMResult, "Failed to get position\n");
   NumSamples = MMTime.u.cb;
	RETAILMSG(1,(TEXT("\r\ngesampelt ... %d\r\n"), NumSamples ));

   // Gracefully stop
   MMResult = waveInReset(WaveInHandle);
   CheckError(MMResult, "Failed to reset\n");
   MMResult = waveInUnprepareHeader(WaveInHandle, &WaveHeader, sizeof(WaveHeader));
   CheckError(MMResult, "Failed to unprepare header");
   MMResult = waveInClose(WaveInHandle);
   CheckError(MMResult, "Failed to close input");

   return true;
}

bool Play(char *buffer, int NumSamples)
{
  HWAVEOUT WaveHandle;
  WAVEHDR WaveHeader;
  MMRESULT MMResult;
  WAVEFORMATEX WaveFormat;

  // Setup the WaveFormat structure and open the Output device
  WaveFormat.wFormatTag =		WAVE_FORMAT_PCM;
  WaveFormat.nChannels =		2;
  WaveFormat.wBitsPerSample =	16;
  WaveFormat.nSamplesPerSec =	SAMPLE_RATE;
  WaveFormat.nBlockAlign =		( WaveFormat.nChannels * WaveFormat.wBitsPerSample ) / 8;
  WaveFormat.nAvgBytesPerSec =	WaveFormat.nBlockAlign * WaveFormat.nSamplesPerSec;
  
  WaveFormat.cbSize =			0;
  MMResult = waveOutOpen(&WaveHandle, WAVE_MAPPER, &WaveFormat,  0, 0, WAVE_FORMAT_QUERY);
	CheckError(MMResult, "Failed to open wave out query");

  MMResult = waveOutOpen(&WaveHandle, WAVE_MAPPER, &WaveFormat, 0, 0, CALLBACK_EVENT);
	CheckError(MMResult, "Failed to open wave out");

  // Setup the WaveHeader, whcih contaions info about the data to play
  memset(&WaveHeader, 0, sizeof(WaveHeader));
  WaveHeader.lpData = buffer;
  WaveHeader.dwBufferLength = NumSamples * 4;
  WaveHeader.dwLoops = 1;
  WaveHeader.dwFlags = WHDR_BEGINLOOP | WHDR_ENDLOOP;

  MMResult = waveOutPrepareHeader(WaveHandle, &WaveHeader, sizeof(WaveHeader));
  CheckError(MMResult, "Failed to prepare header");

  // "write" (ie play) the samples
  MMResult = waveOutWrite(WaveHandle, &WaveHeader, sizeof(WaveHeader));
  CheckError(MMResult, "Failed to write");

  // Wait until done
  printf("Playing");
  fflush(stdout);
  Sleep(1000*NumSamples/SAMPLE_RATE);
  while(!(WaveHeader.dwFlags & WHDR_DONE))
    Sleep(10);
  printf("\n");

  // Gracefully shutdown
  MMResult = waveOutUnprepareHeader(WaveHandle, &WaveHeader, sizeof(WaveHeader));
  CheckError(MMResult, "Failed to unprepare header");
  MMResult = waveOutClose(WaveHandle);
  CheckError(MMResult, "Failed to close");

  return true;
}


const unsigned char hdr[] = {0x52,0x49,0x46,0x46,0x24,0xe2,0x04,0x00,0x57,0x41,0x56,0x45,0x66,0x6d,0x74,0x20,0x10,
							 0x00,0x00,0x00,0x01,0x00,0x02,0x00,0x40,0x1f,0x00,0x00,0x00,0x7d,0x00,0x00,0x04,0x00,
							 0x10,0x00,0x64,0x61,0x74,0x61,0x00,0xe2,0x04,0x00 };

							 
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	int NumSamples = 10*SAMPLE_RATE;
	char *Buffer = (char *)malloc(10*SAMPLE_RATE*4);



	RETAILMSG(1,(TEXT("RECORD ::: \r\n")));
	RETAILMSG(1,(TEXT("     NumSamples  = %d \r\n"), NumSamples));
	RETAILMSG(1,(TEXT("     SAMPLE_RATE = %d \r\n"), SAMPLE_RATE));
	RETAILMSG(1,(TEXT("     Buffer      = 0x%x \r\n"), Buffer ));
	
	if(Record(Buffer, NumSamples)) {
		FILE *f = fopen("\\test.wav", "wb+");
		if ( INVALID_HANDLE_VALUE != f ) {
			// pc like schreiben
			fwrite(hdr, sizeof(hdr), 1, f);
			fwrite(Buffer, 10*SAMPLE_RATE*4, 1, f);
			fclose(f);
		}
		Play(Buffer, NumSamples);
	}
	free(Buffer);

	return 0;
}

