//------------------------------------------------------------------------------
//
//  Copyright (C) 2009-2010, Freescale Semiconductor, Inc. All Rights Reserved.
//  THIS SOURCE CODE, AND ITS USE AND DISTRIBUTION, IS SUBJECT TO THE TERMS
//  AND CONDITIONS OF THE APPLICABLE LICENSE AGREEMENT
//
//------------------------------------------------------------------------------
// UpdateImage.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "UpdateImage.h"
#include "UpdateImageDlg.h"
#include "updatesb.h"
#include "common_nandfmd.h"
#include "winioctl.h"
#include "ioctl_cfg.h"
#include <storemgr.h>
#include "sdmmc.h"

#define IOCTL_HAL_REBOOT                        CTL_CODE(FILE_DEVICE_HAL, 15, METHOD_BUFFERED, FILE_ANY_ACCESS)
extern "C" BOOL KernelIoControl(DWORD dwIoControlCode, LPVOID lpInBuf, DWORD nInBufSize, LPVOID lpOutBuf, DWORD nOutBufSize, LPDWORD lpBytesReturned);


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

extern HANDLE hThread;
// CUpdateImageApp

BootMode    g_bm;
typedef HANDLE (*PFN_FindFirstStore)(PSTOREINFO pStoreInfo);
typedef BOOL (*PFN_FindNextStore)(HANDLE hSearch,PSTOREINFO pStoreInfo);

BEGIN_MESSAGE_MAP(CUpdateImageApp, CWinApp)
END_MESSAGE_MAP()

HANDLE FindAndOpenDSK(LPTSTR Profile, DWORD dwSize, PSTOREINFO pSI)
{
    HANDLE hStore;
    BOOL bResult = TRUE;
    HINSTANCE hins = NULL;
    PFN_FindFirstStore pFindFirstStore;
    PFN_FindNextStore pFindNextStore;

    hins = LoadLibrary(L"coredll.dll");
    
    if(!hins)
    {
        ERRORMSG(TRUE, (_T("LoadLibrary coredll.dll failed")));
        return NULL;
    }
    else
    {
        pFindFirstStore =(PFN_FindFirstStore)GetProcAddress(hins, L"FindFirstStore");
        pFindNextStore = (PFN_FindNextStore)GetProcAddress(hins, L"FindNextStore");
        if(!pFindFirstStore || !pFindNextStore)
        {
            ERRORMSG(TRUE, (_T("GetProcAddress failed")));
            FreeLibrary(hins);
            return NULL;
        }
    }
    
    // 1. find the correct device
    pSI->cbSize = sizeof(STOREINFO);
    hStore = pFindFirstStore(pSI);
    
    if(!hStore)
    {
        ERRORMSG(TRUE, (_T("Failed to find first store\r\n")));
        FreeLibrary(hins);
        return NULL;
    }
    
    while(memcmp(CharUpper(pSI->sdi.szProfile), Profile, dwSize))
    {
        if(!pFindNextStore(hStore, pSI))
        {
            bResult = FALSE;
            break;
        }
    }    
    
    if(!bResult)
    {
        //ERRORMSG(TRUE, (_T("Failed to find the store: %s\r\n"),Profile));
        CloseHandle(hStore);
        FreeLibrary(hins);
        return NULL;
    }
    RETAILMSG(TRUE, (_T("find the target store: %s\r\n"), pSI->sdi.szProfile));
    RETAILMSG(TRUE, (_T("DeviceName=%s\r\n"), pSI->szDeviceName));
    RETAILMSG(TRUE, (_T("dwBytesPerSector=0x%x\r\n"), pSI->dwBytesPerSector));
    
    CloseHandle(hStore);
    FreeLibrary(hins);
    
    // 2. Open the device
    return CreateFile(pSI->szDeviceName,            // name of device
                    GENERIC_READ|GENERIC_WRITE,         // desired access
                    FILE_SHARE_READ|FILE_SHARE_WRITE,   // sharing mode
                    NULL,                               // security attributes (ignored)
                    OPEN_EXISTING,                      // creation disposition
                    FILE_FLAG_RANDOM_ACCESS,            // flags/attributes
                    NULL);                              // template file (ignored)
}
BOOL CreateSign()
{
    DWORD sign = UPDATE_SIGN;
    
    if(!KernelIoControl(IOCTL_HAL_SET_UPDATE_SIG, NULL, 0, &sign, sizeof(sign), NULL))
    {
        ERRORMSG(1, (TEXT("IOCTL_HAL_SET_UPDATE_SIG failed!\r\n")));
        return FALSE;
    }
    return TRUE;
}

void RebootSystem(BOOL FirstBoot, CUpdateImageDlg *dlg)
{
    BootMode boot;
    
    boot.source = FirstBoot == TRUE ? First : Second;
    
    if(!KernelIoControl(IOCTL_HAL_SET_BOOT_SOURCE, NULL, 0, &boot, sizeof(BootMode), NULL))
    {
        ERRORMSG(1, (TEXT("IOCTL_HAL_SET_BOOT_SOURCE failed!\r\n")));
        return;
    }
    
    RETAILMSG(1, (_T("rebooting... ...\r\n")));

    dlg->UpdateData(); 
    dlg->m_static.SetWindowTextW(L"system will reboot after 5 seconds");
    dlg->m_static1.SetWindowTextW(L" ");
    dlg->UpdateData(FALSE);
    Sleep(1000);
    dlg->UpdateData(); 
    dlg->m_static.SetWindowTextW(L"system will reboot after 4 seconds");
    dlg->UpdateData(FALSE);
    Sleep(1000);
    dlg->UpdateData(); 
    dlg->m_static.SetWindowTextW(L"system will reboot after 3 seconds");
    dlg->UpdateData(FALSE);
    Sleep(1000);
    dlg->UpdateData(); 
    dlg->m_static.SetWindowTextW(L"system will reboot after 2 seconds");
    dlg->UpdateData(FALSE);
    Sleep(1000);
    dlg->UpdateData(); 
    dlg->m_static.SetWindowTextW(L"system will reboot after 1 seconds");
    dlg->UpdateData(FALSE);
    Sleep(1000);
    dlg->UpdateData(); 
    dlg->m_static.SetWindowTextW(L"system will reboot after 0 seconds");
    dlg->UpdateData(FALSE);
    Sleep(1000);

    KernelIoControl(IOCTL_HAL_REBOOT, NULL, 0, NULL, 0, NULL);
}

void SDMMCBootUpdate(CUpdateImageDlg *dlg)
{
    HANDLE hDSK = NULL, hFile = NULL;
    SG_REQ sq;
    STOREINFO si;
    DWORD dwUpdateImgLenth;
    
    // 1. find the correct device
    hDSK = FindAndOpenDSK(L"SDMEMORY", sizeof(L"SDMEMORY"), &si);
    
    if(!hDSK)
    {
        hDSK = FindAndOpenDSK(L"MMC", sizeof(L"MMC"), &si);
        if(!hDSK)
        {
            ERRORMSG(TRUE, (_T("failed to find and open store\r\n")));
            return;
        }
    }
    
    // 2. read MBR from device
    memset(&sq, 0, sizeof(SG_REQ));
    
    sq.sr_start = 0;
    sq.sr_num_sec = 1;
    sq.sr_num_sg = 1;
    sq.sr_sglist[0].sb_buf = (PBYTE)LocalAlloc(LMEM_ZEROINIT, BUFFER_SIZE);
    sq.sr_sglist[0].sb_len = BUFFER_SIZE;
    
    if(!sq.sr_sglist[0].sb_buf)
    {
        ERRORMSG(TRUE, (_T("alloc memory failed\r\n")));   
        goto exit;   
    }
    
    if (!DeviceIoControl(hDSK, IOCTL_DISK_READ, &sq, sizeof(SG_REQ), NULL, 0, NULL, NULL))
    {
        ERRORMSG(TRUE, (_T("IOCTL_DISK_READ failed\r\n")));   
        goto exit;
    }
    sq.sr_num_sec = BUFFER_SIZE / si.dwBytesPerSector;
    
    // 3. Parse MBR
    PMASTERBOOT_RECORD pMBR = (PMASTERBOOT_RECORD)sq.sr_sglist[0].sb_buf;
    DWORD dwBootOffset, dwBootSize, i;
    
    if(pMBR->Signature == 0xAA55)
    {
        for(i = 0; i < MAX_PARTTABLE_ENTRIES; i++)
        {
            if(pMBR->Partition[i].SystemId == 'S')       //soc spec   
            {
                //it's boot partition
                break;
            }
        }
        
        if(i == MAX_PARTTABLE_ENTRIES)
        {
            ERRORMSG(TRUE, (_T("Not MBR format, skipping\r\n")));
            goto exit;
        }
        else
        {
            dwBootOffset = (pMBR->Partition[i].RelativeSector + 4 + 3) & ~3;   //sector addr must be 4 aligned
            dwBootSize = (pMBR->Partition[i].TotalSector - (dwBootOffset - pMBR->Partition[i].RelativeSector)) & ~7;
        }
    }
    
    hFile = CreateFile(dlg->m_filename.GetString(),            // name of device
                    GENERIC_READ,                       // desired access
                    0,                                  // non-sharing mode
                    NULL,                               // security attributes (ignored)
                    OPEN_EXISTING,                      // creation disposition
                    FILE_FLAG_RANDOM_ACCESS,            // flags/attributes
                    NULL);                              // template file (ignored)
    if(INVALID_HANDLE_VALUE == hFile)
    {
        RETAILMSG(TRUE, (_T("can't find updated file\r\n")));  
        goto exit;
    }          
    dwUpdateImgLenth = GetFileSize(hFile, NULL);
    if(dwUpdateImgLenth > dwBootSize / 2 * si.dwBytesPerSector)
    {
        ERRORMSG(TRUE, (_T("file size(0x%x) too big, should be less than 0x%x\r\n"),dwUpdateImgLenth, dwBootSize / 2)); 
        goto exit;
    }
    
    if(!CreateSign())
    {
        RETAILMSG(1, (_T("Please make sure your SD/MMC card is writable!\r\n")));
        goto exit;
    }
    
    {
        DWORD NumberOfBytesRead, index = 0;
        BOOL bLastPacket = FALSE;
        
        dlg->m_process.SetRange(0, 100);
        dlg->m_process.SetPos(0);
        
        sq.sr_start = dwBootOffset + dwBootSize / 2;
        RETAILMSG(1, (_T("INFO: sq.sr_start 0x%x\r\n"), sq.sr_start));
        while(!bLastPacket && (index * sq.sr_sglist[0].sb_len != dwUpdateImgLenth))
        {
            memset(sq.sr_sglist[0].sb_buf, 0, sq.sr_sglist[0].sb_len);
            
            if(!ReadFile(hFile, sq.sr_sglist[0].sb_buf, sq.sr_sglist[0].sb_len, &NumberOfBytesRead, NULL))
            {
                ERRORMSG(TRUE, (_T("ReadFile failed\r\n")));   
                goto exit;
            }
            else if(NumberOfBytesRead != sq.sr_sglist[0].sb_len)
            {
                RETAILMSG(TRUE, (_T("bLastPacket!\r\n"))); 
                sq.sr_num_sec = (NumberOfBytesRead + si.dwBytesPerSector - 1) / si.dwBytesPerSector;
                bLastPacket = TRUE;
            }
            
            if (!DeviceIoControl(hDSK, IOCTL_DISK_WRITE, &sq, sizeof(SG_REQ), NULL, 0, NULL, NULL))
            {
                ERRORMSG(TRUE, (_T("IOCTL_DISK_WRITE failed\r\n")));   
                goto exit;
            }
            
            dlg->m_process.SetPos(100 * (index * sq.sr_sglist[0].sb_len + NumberOfBytesRead) / dwUpdateImgLenth);
            sq.sr_start += sq.sr_num_sec;
            index++;
            RETAILMSG(1, (_T("INFO: sq.sr_start 0x%x\r\n"), sq.sr_start));
        }
    }
    
    RebootSystem(FALSE, dlg);
exit:
    if(sq.sr_sglist[0].sb_buf)
        LocalFree(sq.sr_sglist[0].sb_buf);
    if(hDSK)
        CloseHandle(hDSK);    
    if(hFile)
        CloseHandle(hFile);              
    return;
}

void NANDBootUpdate(CUpdateImageDlg *dlg)
{
    HANDLE hDSK = NULL, hFile = NULL;
    STOREINFO si;
    NANDImgInfo ImgInfo = {};
    DWORD dwUpdateImgLenth;
    PBYTE UpdateImg = NULL;
    
    // 1. find the correct device
    hDSK = FindAndOpenDSK(L"MSFLASH", sizeof(L"MSFLASH"), &si);
    
    if(!hDSK)
    {
        ERRORMSG(TRUE, (_T("failed to find and open store\r\n")));
        return;
    }
        
    if (!DeviceIoControl(hDSK,      // file handle to the driver
        IOCTL_DISK_VENDOR_GET_SBIMGINFO,  // I/O control code
        NULL,                       // in buffer
        0,                          // in buffer size
        &ImgInfo,                   // out buffer
        sizeof(NANDImgInfo),        // out buffer size
        NULL,                       // pointer to number of bytes returned
        NULL))                      // ignored (=NULL)
    {
        ERRORMSG(1, (TEXT("IOCTL_DISK_VENDOR_GET_SBIMGINFO failed!\r\n")));
        goto exit;
    }
    hFile = CreateFile(dlg->m_filename.GetString(),            // name of device
                    GENERIC_READ|GENERIC_WRITE,         // desired access
                    0,                                  // non-sharing mode
                    NULL,                               // security attributes (ignored)
                    OPEN_EXISTING,                      // creation disposition
                    FILE_FLAG_RANDOM_ACCESS,            // flags/attributes
                    NULL);                              // template file (ignored)
    if(INVALID_HANDLE_VALUE == hFile)
    {
        RETAILMSG(TRUE, (_T("can't find updated file\r\n")));  
        goto exit;
    }              
    
    dwUpdateImgLenth = GetFileSize(hFile, NULL);
    if(dwUpdateImgLenth > ImgInfo.dwSBLen)
    {
        ERRORMSG(TRUE, (_T("file size(0x%x) too big, should be less than 0x%x\r\n"),dwUpdateImgLenth, ImgInfo.dwSBLen)); 
        goto exit;
    }
    RETAILMSG(1, (_T("Begin to update firmware, don't remove SD/MMC card during updating\r\n")));
    
    UpdateImg = (PBYTE)LocalAlloc(LMEM_ZEROINIT, ImgInfo.dwSBLenUnit);
    
    if(!UpdateImg)
    {
        ERRORMSG(TRUE, (_T("LocalAlloc MEMORY FAILED")));
        goto exit;   
    }

    if(!CreateSign())
    {
        RETAILMSG(1, (_T("Please make sure your SD/MMC card is writable!\r\n")));
        goto exit;
    }
    
    {
        DWORD NumberOfBytesRead, index = 0;
        BOOL bLastPacket = FALSE;
        NANDIORequest request;
        dlg->m_process.SetRange(0, 100);
        dlg->m_process.SetPos(0);

        while(!bLastPacket)
        {
            if(!ReadFile(hFile, UpdateImg, ImgInfo.dwSBLenUnit, &NumberOfBytesRead, NULL))
            {
                goto exit;
            }
            else if(NumberOfBytesRead != ImgInfo.dwSBLenUnit)
            {
                bLastPacket = TRUE;
            }
            
            request.dwIndex = index;
            request.SBPos = SecondBoot;
    
            dlg->m_process.SetPos(100 * (index * ImgInfo.dwSBLenUnit + NumberOfBytesRead)    / dwUpdateImgLenth);

            if (!DeviceIoControl(hDSK,      // file handle to the driver
                IOCTL_DISK_VENDOR_WRITE_SB,  // I/O control code
                &request,                   // in buffer
                sizeof(NANDIORequest),      // in buffer size
                UpdateImg,                  // out buffer
                ImgInfo.dwSBLenUnit,        // out buffer size
                NULL,                       // pointer to number of bytes returned
                NULL))                      // ignored (=NULL)
            {
                ERRORMSG(1, (TEXT("IOCTL_DISK_VENDOR_WRITE_SB failed!\r\n")));
                goto exit;
            }
            index++;
            
            if(bLastPacket)
            {
                if (!DeviceIoControl(hDSK,          // file handle to the driver
                    IOCTL_DISK_VENDOR_END_WRITE_SB,  // I/O control code
                    &request,                       // in buffer
                    sizeof(NANDIORequest),          // in buffer size
                    NULL,                           // out buffer
                    0,                              // out buffer size
                    NULL,                           // pointer to number of bytes returned
                    NULL))                          // ignored (=NULL)
                {
                    ERRORMSG(1, (TEXT("IOCTL_DISK_VENDOR_WRITE_SB failed!\r\n")));
                    goto exit;
                }
            }
        }
    }

    RebootSystem(FALSE, dlg);
exit:
    if(UpdateImg)
        LocalFree(UpdateImg);
    if(hDSK)
        CloseHandle(hDSK);    
    if(hFile)
        CloseHandle(hFile);                
}

BOOL DoUpdateSB(CUpdateImageDlg *dlg)
{
    RETAILMSG(1,(_T("FilePath=%s\r\n"),dlg->m_filename.GetString()));
    
    if(!KernelIoControl(IOCTL_HAL_QUERY_BOOT_MODE, NULL, 0, &g_bm, sizeof(g_bm), NULL))
    {
        ERRORMSG(1, (TEXT("IOCTL_HAL_QUERY_BOOT_MODE failed!\r\n")));
        goto exit;
    }
    
    switch(g_bm.media)
    {
        case NandBoot:
            RETAILMSG(1, (_T("booted from NAND Flash\r\n")));
            break;
        case SDMMCBoot:
            RETAILMSG(1, (_T("booted from SDMMCBoot\r\n")));
            break;
        case SPIBoot:
            RETAILMSG(1, (_T("booted from SPI, skip\r\n")));
            goto exit;
        default:
            RETAILMSG(1, (_T("booted from UNKNOWN, skip\r\n")));
            goto exit;  
    }
    RETAILMSG(1, (_T("booted from %s\r\n"),g_bm.source == First? L"First Boot" : L"Second Boot"));
    
    if(g_bm.media == SDMMCBoot)
    {
        SDMMCBootUpdate(dlg);
    }
    
    if(g_bm.media == NandBoot)
    {
        NANDBootUpdate(dlg);
    }
    
exit:
    return TRUE;
}
// CUpdateImageApp construction
CUpdateImageApp::CUpdateImageApp()
    : CWinApp()
{
    // TODO: add construction code here,
    // Place all significant initialization in InitInstance
}


// The one and only CUpdateImageApp object
CUpdateImageApp theApp;

// CUpdateImageApp initialization

BOOL CUpdateImageApp::InitInstance()
{
    HANDLE mutex;
    // SHInitExtraControls should be called once during your application's initialization to initialize any
    // of the Windows Mobile specific controls such as CAPEDIT and SIPPREF.
    SHInitExtraControls();

    // Standard initialization
    // If you are not using these features and wish to reduce the size
    // of your final executable, you should remove from the following
    // the specific initialization routines you do not need
    // Change the registry key under which our settings are stored
    // TODO: You should modify this string to be something appropriate
    // such as the name of your company or organization
    SetRegistryKey(_T("Local AppWizard-Generated Applications"));
    mutex = CreateMutex(NULL, FALSE, MUTEX_NAME);
    if(mutex && GetLastError() == ERROR_ALREADY_EXISTS)
    {
        RETAILMSG(TRUE, (_T("process has already been running\r\n")));
        MessageBox(NULL, L"process has already been running!", L"Message", MB_OK);
        return FALSE;
    }
    CUpdateImageDlg dlg;
    m_pMainWnd = &dlg;
    INT_PTR nResponse = dlg.DoModal();
    if (nResponse == IDOK)
    {
        // TODO: Place code here to handle when the dialog is
        //  dismissed with OK
    }
    if(mutex)
        CloseHandle(mutex);
    // Since the dialog has been closed, return FALSE so that we exit the
    //  application, rather than start the application's message pump.
    return FALSE;
}
